/*
  Switchless warm/cold reset system for the Commodore 64 and optional rom selector
  Based on Version 1.01 by By MtnBuffalo/breadbox64.com 2017(updated October 18th 2017)
  For original project visit: http://www.breadbox64.com/blog/switchless-reset-mod/
  
  Function:
  If RESTORE button is pressed for 2-4 seconds -> RESET line LOW (Warm Reset)
  If RESTORE button is pressed for 2-4 seconds -> RESET line LOW & /EXROM line LOW for 300ms extra (Cold Reset)
  If RESTORE button is pressed for 7-10 seconds -> HARD RESET + switch rom address to negated previous state
  If RESTORE button is pressed for >10 seconds -> nothing happens
  If RESTORE button is pressed during POWER ON -> Startum using the second 
  
  Disclamier:
  Tested on short boards (Assy 250469) & long boards (Assy 250425)
  Assumed to work across all motherboard versions
  Use at your own risk!
 
  Credits:
  Base project by MtnBuffalo/breadbox64.com,
  Pinmode of RESET line is held as INPUT until requested to pull it LOW during Reset. This is to keep the Arduino
  from constantly driving the reset line, suggested by bwack.
  Cartrige compatibility fix and Rom Selector functionality by Gohanks
*/
 
// Constants
const int RESTORE_button = A0;        // pin A0 is connected to the RESTORE button (analog input pin)
const int RESET_line = 3;             // pin 3 is connected to the RESET line (digital output pin)
const int EXROM_line = 4;             // pin 4 is connected to the /EXROM line of the PLA (digital output pin)
const int ROMSEL_line = 5;            // pin 5 is connected to pin 27 of an EEPROM (27256) DUALKERNAL
const int press_time_warm = 2000;     // time in milliseconds before a warm reset is initiated
const int press_time_cold = 4000;     // time in milliseconds before a cold reset is initiated
const int press_time_romsel = 7000;   // time in milliseconds before a cold reset and rom selection switching
const int max_polling_time = 10000;   // time in milliseconds after which nothing will happen
const int voltageValue = 100;         // ADC value that will trigger the timer (voltages less than 0.5V) -> RESTORE pushed 
 
// Variables
unsigned long RESTORE_press_time = 0; // the time the RESTORE button is being pressed in milliseconds
unsigned long StartTime = 0;          // first time stamp for calculating RESTORE_press_time
int firstpress = 1;                   // test if the RESTORE button has been pressed
int analogValue;                      // ADC value (0-1023) read from the RESTORE button
int firstRom = 0;                     // Indicates which rom should load after power on
 
// Functions
int debounce();                       // debounce function to get ADC value (analogValue)
 
void setup()
{
  //initialize digital pins
  pinMode(RESET_line, INPUT);         // digital pin 3 on the Arduino Pro Mini - Set as OUTPUT before reset 
  pinMode(EXROM_line, INPUT);        // digital pin 4 on the Arduino Pro Mini
  pinMode(ROMSEL_line, OUTPUT);       // digital pin 5 on the Arduino Pro Mini
  
  digitalWrite(RESET_line, HIGH);     // initial RESET_line state
  digitalWrite(EXROM_line, HIGH);     // initial EXROM_line state
  digitalWrite(ROMSEL_line, LOW);    // initial ROMSEL_line state  
  
  pinMode(RESTORE_button, INPUT);     // analog pin A0 on the Arduino Pro Mini
  analogReference(DEFAULT);           // setting the analog reference to 5V
  
  analogValue = debounce();                       // call debounce function to get average ADC value
  if(analogValue < voltageValue)                  // checks if the RESTORE button has been pressed
  {
    firstRom = 1;
    digitalWrite(ROMSEL_line, HIGH);    // initial ROMSEL_line state
  } else {
    firstRom = 0;
    digitalWrite(ROMSEL_line, LOW);    // initial ROMSEL_line state
  }
}
 
void loop() 
{
  analogValue = debounce();                       // call debounce function to get average ADC value
  if(analogValue < voltageValue)                  // checks if the RESTORE button has been pressed
  {
    // initiate RESTORE push timer
    if(firstpress == 1)                       
    {
      firstpress = 0;                   
      StartTime = millis();          // first time stamp
      delay(2);                      // make sure that RESTORE_press_time will become > 0 milliseconds
    }
    RESTORE_press_time = millis() - StartTime;   // variable holds total time RESTORE is pressed
  }
  else if(firstpress == 0) firstpress = 1;       // RESTORE button has been released
   
  // Warm reset
  
  if (firstpress == 1 && RESTORE_press_time > press_time_warm && RESTORE_press_time < press_time_cold)
  {
    pinMode(RESET_line, OUTPUT);                  // set as output to pull down the reset line
    digitalWrite(RESET_line, LOW);                // anode side of CR5 diode (Assy 250469) or pin 3 of User Port (Assy 250425) LOW
    delay(200);
    digitalWrite(RESET_line, HIGH);               // reset RESET_line
    RESTORE_press_time = 0;                       // reset timer
    pinMode(RESET_line, INPUT);                   // reset the RESET line
  }  
  
  // Cold reset
  else if (firstpress == 1 && RESTORE_press_time > press_time_cold && RESTORE_press_time < press_time_romsel) //cancel reset if RESTORE is pushed > 10 seconds
  {
    pinMode(RESET_line, OUTPUT);                  // set as OUTPUT to pull down the reset line
    pinMode(EXROM_line, OUTPUT);                  // set as OUTPUT to pull down the EXROM line
    digitalWrite(RESET_line, LOW);                // anode side of CR5 diode (Assy 250469) or pin 3 of User Port (Assy 250425) LOW
    digitalWrite(EXROM_line, LOW);                // /EXROM line of the PLA LOW
    delay(200);
    digitalWrite(RESET_line, HIGH);               // reset RESET_line
    delay(300);
    digitalWrite(EXROM_line, HIGH);               // reset EXROM_line
    
    RESTORE_press_time = 0;                       // reset timer
    pinMode(RESET_line, INPUT);                   // reset the RESET line
    pinMode(EXROM_line, INPUT);
  }

  // Cold reset and rom select
  else if (firstpress == 1 && RESTORE_press_time > press_time_romsel && RESTORE_press_time < max_polling_time) 
  {
    pinMode(RESET_line, OUTPUT);                  // set as OUTPUT to pull down the reset line
    pinMode(EXROM_line, OUTPUT);                  // set as OUTPUT to pull down the EXROM line
    digitalWrite(RESET_line, LOW);                // anode side of CR5 diode (Assy 250469) or pin 3 of User Port (Assy 250425) LOW
    digitalWrite(EXROM_line, LOW);                // /EXROM line of the PLA LOW
    if(firstRom == 1)
    {
      digitalWrite(ROMSEL_line, LOW);             // switch to first ROM
      firstRom = 0;
    } 
    else 
    {
      digitalWrite(ROMSEL_line, HIGH);            // switch to second ROM
      firstRom = 1;
    }    
    delay(200);
    digitalWrite(RESET_line, HIGH);               // reset RESET_line
    delay(300);
    digitalWrite(EXROM_line, HIGH);               // reset EXROM_line
    
    RESTORE_press_time = 0;                       // reset timer
    pinMode(RESET_line, INPUT);                   // reset the RESET line
    pinMode(EXROM_line, INPUT);
  }
}
 
// Debounce function to filter transistions/noise of the analog reading
int debounce()
{
  int x, meanADC, sum = 0;
  for (int i = 0; i < 5; i++){                                            // reads ADC value from pin A0 5 times
     delay(5);                                                            // debounce for 25 ms total
     sum += analogRead(RESTORE_button);
  }
  x = sum / 5;                                                            // mean ADC value  
 (x < voltageValue) ? (meanADC = x) : (meanADC = 1023);                   // if/else check - has RESTORE been pushed?
  return meanADC;                                                         // return mean ADC value to main loop
}
